@@ -163,7 +163,7 @@ os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"  # FATAL
 logging.getLogger("tensorflow").setLevel(logging.FATAL)
 
 
-class GAN(object):
+class GAN():
     """Implements a generative adversarial net (GAN) learning model.
 
         Typical usage example:
