@@ -30,13 +30,13 @@ import logging
 import os
 import time
 from datetime import datetime
-import numpy as np
 
 import tensorflow as tf
 
 from dnnlab.errors.dnnlab_exceptions import ModelNotCompiledError
 from dnnlab.losses import yolo_loss
 from dnnlab.losses import extract_model_output, extract_label, yolo_grid
+from dnnlab.utils.coco_map import compute_coco_metrics
 
 os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"  # FATAL
 logging.getLogger("tensorflow").setLevel(logging.FATAL)
@@ -54,22 +54,31 @@ class YOLO(object):
                              but num_channels can vary.
 
         # Define forward path.
-        grid_size = 7
-        n_bounding_boxes = 2
-        class_list = ["foo", "bar"]
-        feature_extractor = tf.keras.models.Sequential/Custom/Functional
-        yolo = YOLO(feature_extractor, grid_size, n_bounding_boxes, n_classes)
+        # TODO: only allows symmetric grid.
+        GRID_H = 13
+        GRID_W = 13
+        ANCHORS = np.array([
+                        4.0, 2.0,
+                        2.0, 4.0,
+                        2.0, 2.0,
+                        8.0, 1.0
+                    ])
+        LABELS = ["foo", "bar"]
+
+        feature_extractor = tf.keras.models....
+        yolo = dnnlab.image_detection.YOLO(feature_extractor, GRID_H, GRID_W,
+                                       ANCHORS, LABELS)
 
         # Define optimizer.
         yolo.compile(optimizer="adam", lr=1e-4)
 
         # Start training process.
-        yolo.fit(x,y,x_val,y_val, EPOCHS, BATCH_SIZE, save_ckpt=5)
+        yolo.fit(training_data, validation_data, EPOCHS, BATCH_SIZE,LEN_DATASET)
 
-        # Export both models.
+        # Export h5 model.
         yolo.export()
 
-        use gan.restore("relative_path_to_logs") to continue training after
+        use yolo.restore("relative_path_to_logs") to continue training after
         a break.
 
 
@@ -77,8 +86,12 @@ class YOLO(object):
         feature_extractor (keras.model): Basic nn feature extractor.
         grid_h (int): Grid size.
         grid_w (int): Grid size.
-        anchors (int): Number of bounding boxes per grid cell.
-        n_classes (int): Number of different objects.
+        anchors (np.array): List of w,h of prior bounding boxes.
+        boxes(int): Number of prior defined bounding boxes.
+        class_list (int): Labels as a list of strings.
+        n_classes(int): Number of different classes.
+        optimizer(keras.optimizers): Learning algorithm instance.
+        complete_model(keras.model): Feature extractor + classification head.
         init_timestamp (str): Acts as a unique folder identifier.
         logdir (str): Top level logdir.
         tensorboard (str): Path to tensorboard summary files.
@@ -86,7 +99,9 @@ class YOLO(object):
         ckpt_manager (tf.train.CheckpointManager): Deletes old checkpoints.
         checkpoint (tf.train.Checkpoint): Groups trackable objects, saving and
             restoring them.
+        colors (tf.constant): Color of predicted bounding boxes.
     """
+
     def __init__(self,
                  feature_extractor,
                  grid_h,
@@ -94,12 +109,20 @@ class YOLO(object):
                  anchors,
                  class_list,
                  complete_model=None):
-        """TODO
+        """Inits YOLO with a predefined keras model. A complete model is a
+        neural net consisting of a feature extractor (body) and a yolo
+        classification head. If only a feature extractor is given YOLO will
+        automatically append a corresponding classification head depending on
+        a grid size, the number of prior anchor boxes and the number of classes.
 
         Args:
-        The more bounding boxes, the more objects can be detected in close
-        neighborhood.
-
+            feature_extractor (keras.model): Basic nn feature extractor.
+            grid_h (int): Grid size.
+            grid_w (int): Grid size.
+            anchors (np.array): List of w,h of prior bounding boxes.
+            class_list (list): Labels as a list of strings.
+            complete_model (keras.model, optional): Feature extractor +
+                classification head.. Defaults to None.
         """
         self.feature_extractor = feature_extractor
         self.grid_h = grid_h
@@ -108,14 +131,14 @@ class YOLO(object):
         self.boxes = len(anchors) // 2
         self.class_list = class_list
         self.n_classes = len(class_list)
+        self.category_list = self._list_to_dict()
         self.optimizer = None
         self.complete_model = complete_model
         if self.complete_model is None:
             self.model = self._append_classifier()
         else:
             self.model = self.complete_model
-        self.init_timestamp = "YOLO-" + datetime.now().strftime(
-            "%d%m%Y-%H%M%S")
+        self.init_timestamp = "YOLO-" + datetime.now().strftime("%d%m%Y-%H%M%S")
         self.logdir = os.path.join("logs", self.init_timestamp)
         self.tensorboard = os.path.join(self.logdir, "tensorboard")
         self.ckpt_dir = os.path.join(self.logdir, "ckpts")
@@ -123,7 +146,14 @@ class YOLO(object):
         self.checkpoint = None
         self.colors = tf.constant([(253, 165, 15)], dtype=tf.float32) / 255.
 
+    def _list_to_dict(self):
+        category_list = []
+        for counter, item in enumerate(self.class_list):
+            category_list.append({"id": counter, "name": item})
+        return category_list
+
     def summary(self):
+        """Returns a keras model summary on it underlying yolo model."""
         return self.model.summary()
 
     def predict(self,
@@ -132,10 +162,25 @@ class YOLO(object):
                 max_number_bb=100,
                 iou_threshold=0.4,
                 class_conf_threshold=0.25):
-        # TODO readable output
-        # non-max surpression
-        # confidence threshold
-        # + visualization
+        """Performs an inference step (prediction) with yolo specific
+        post-processing. After the prediction, the output is filtered by
+        class_confidence below class_conf_threshold:
+            class_confidence = box_confidence * class probability
+        and a multiclass non-max suppression with iou_threshold and a maximal
+        number of returning boundind boxes.
+        If text_format is true, this method returns a cl output TODO.
+        If text_format is false, this method return the input image with
+        its predicted bounding boxes.
+
+        Args:
+            img: Image to perform prediction on.
+            text_format (bool, optional): Ouput format. Defaults to True.
+            max_number_bb (int, optional): Maximum of predicted bounding boxes.
+                Defaults to 100.
+            iou_threshold (float, optional): Used in NMS. Defaults to 0.4.
+            class_conf_threshold (float, optional): Used before NMS.
+                Defaults to 0.25.
+        """
         prediction = self.model.predict(img)
         selected_boxes, selected_confidence, selected_classes = self.decode_yolo_output(
             prediction,
@@ -152,34 +197,37 @@ class YOLO(object):
                     cl, dtype=tf.int32)]))
                 print("Box: {}".format(box))
                 print("")
+                # TODO consitent return.
         else:
-
             # Swap xy coordinates for bullshit tensorflow draw_bounding_boxes
-            x1 = selected_boxes[..., 0:1]
-            y1 = selected_boxes[..., 1:2]
-            x2 = selected_boxes[..., 2:3]
-            y2 = selected_boxes[..., 3:4]
+            xmin = selected_boxes[..., 0:1]
+            ymin = selected_boxes[..., 1:2]
+            xmax = selected_boxes[..., 2:3]
+            ymax = selected_boxes[..., 3:4]
 
-            boxes_yx = tf.concat([y1, x1, y2, x2], axis=-1)
+            boxes_yx = tf.concat([ymin, xmin, ymax, xmax], axis=-1)
 
             # Append batch dim.
             boxes_yx = tf.cast(tf.expand_dims(boxes_yx, 0), dtype=tf.float32)
 
-            return tf.image.draw_bounding_boxes(tf.cast(img, dtype=tf.float32),
-                                                boxes_yx, self.colors)
+            return tf.image.draw_bounding_boxes(
+                tf.cast(img, dtype=tf.float32), boxes_yx, self.colors)
 
     def _append_classifier(self):
         self.feature_extractor.add(
             # 1x1 convolution.
-            tf.keras.layers.Conv2D(filters=(self.boxes * (5 + self.n_classes)),
-                                   kernel_size=(1, 1),
-                                   strides=(1, 1),
-                                   padding="same"))
+            tf.keras.layers.Conv2D(
+                filters=(self.boxes * (5 + self.n_classes)),
+                kernel_size=(1, 1),
+                strides=(1, 1),
+                padding="same",
+                name="classification_head"))
+
         # Reshape to specific yolov2 output:
         # (bs, grid, grid, anchors, (x,y,w,h,objectivness, n_classes))
         self.feature_extractor.add(
-            tf.keras.layers.Reshape(
-                (self.grid_h, self.grid_w, self.boxes, 5 + self.n_classes)))
+            tf.keras.layers.Reshape((self.grid_h, self.grid_w, self.boxes,
+                                     5 + self.n_classes)))
 
         return self.feature_extractor
 
@@ -198,11 +246,10 @@ class YOLO(object):
             self.optimizer = tf.keras.optimizers.Adam(lr)
 
         if self.checkpoint is None:
-            self.checkpoint = tf.train.Checkpoint(optimizer=self.optimizer,
-                                                  model=self.model)
-            self.ckpt_manager = tf.train.CheckpointManager(self.checkpoint,
-                                                           self.ckpt_dir,
-                                                           max_to_keep=5)
+            self.checkpoint = tf.train.Checkpoint(
+                optimizer=self.optimizer, model=self.model)
+            self.ckpt_manager = tf.train.CheckpointManager(
+                self.checkpoint, self.ckpt_dir, max_to_keep=5)
 
     def fit(self,
             training_data,
@@ -221,12 +268,14 @@ class YOLO(object):
             iou_threshold=0.4,
             class_conf_threshold=0.25,
             mlflow=False):
-        """Trains both models for n EPOCHS. Saves ckpts every n EPOCHS.
+        """Trains the yolo model for n EPOCHS. Saves ckpts every n EPOCHS.
         The training loop together with the optimization algorithm define the
         learning algorithm.
 
         Args:
-            dataset (tf.dataset): tf.Dataset with
+            training_data (tf.dataset): tf.Dataset with
+                shape(None, width, height, depth).
+            validation_data (tf.dataset): tf.Dataset with
                 shape(None, width, height, depth).
             epochs (int): Number of epochs.
             batch_size (int): Batch length.
@@ -234,6 +283,14 @@ class YOLO(object):
             verbose (int, optional): Keras Progbar verbose lvl. Defaults to 1.
             max_outputs (int, optional): Number of images shown in TB.
                 Defaults to 2.
+            initial_step (int, optional): Initial step for tb output.
+            lambda_coords(float, optional): Coordinate weight coefficient.
+            lambda_obj(float, optional): Object weight coefficient.
+            lambda_noobj(float, optional): Noobject weight coefficient.
+            lambda_class(float, optional): Class weight coefficient.
+            iou_threshold(float, optional): For tb nms output.
+            class_conf_threshold(float, optional): For tb nms output.
+            mlflow(bool, optional): Tracks validation loss as metric.
 
         Raises:
             ModelNotCompiledError: Raise if model is not compiled.
@@ -248,27 +305,35 @@ class YOLO(object):
 
         num_batches = len_dataset / batch_size
 
-        # Keras Progbar
+        # Keras Progbar and file writer setup.
         progbar = tf.keras.utils.Progbar(target=num_batches, verbose=verbose)
         file_writer = tf.summary.create_file_writer(self.tensorboard)
         eval_file_writer = tf.summary.create_file_writer(
             os.path.join(self.tensorboard, "eval"))
         file_writer.set_as_default()
+
+        # Iterate over all epocs.
         for epoch in range(epochs):
             step_float = 1
+            train_loss_avg = tf.keras.metrics.Mean()
             start = time.time()
             for elements in training_data:
                 images = elements[0]
                 labels = elements[1]
 
-                tb_prediction, tb_label = self.train_step(
+                # Performs a training step. Inference + BP and GD.
+                tb_prediction, tb_label, loss = self.train_step(
                     images, labels, batch_size, step, max_outputs, file_writer,
                     lambda_coord, lambda_obj, lambda_noobj, lambda_class,
                     iou_threshold, class_conf_threshold)
-                # TODO Workaround.
+
+                train_loss_avg(loss)
+
+                # TODO Workaround. summary image & gpu usage.
                 with file_writer.as_default():
                     tf.summary.image("label", tb_label, step=step)
-                    tf.summary.image("prediction", tb_prediction, step=step)
+                    tf.summary.image(
+                        "prediction_training", tb_prediction, step=step)
 
                 file_writer.flush()
                 progbar.update(current=(step_float))
@@ -281,38 +346,190 @@ class YOLO(object):
                 print("\nSaving checkpoint for epoch {} at {}".format(
                     epoch + 1, ckpt_save_path))
 
-            print(" - Epoch {} finished after {} sec".format(
-                epoch + 1, int(time.time() - start)))
+            print(" - Epoch: {} - loss {}".format(
+                epoch + 1, round(train_loss_avg.result().numpy().item(), 4)))
 
             print("Start validation loop...")
             # Run a validation loop at the end of each epoch.
-            val_loss = []
+            groundtruth_annotations_list = []
+            img_list = []
+            detections_list = []
+            img_id = 1
+            val_loss_avg = tf.keras.metrics.Mean()
             for elements in validation_data:
                 images = elements[0]
                 labels = elements[1]
-                val_logits = self.model(images)
-                # Loss
-                loss = yolo_loss(labels,
-                                 val_logits,
-                                 self.anchors,
-                                 step=step,
-                                 is_training=False)
-                val_loss.append(loss)
-                # Update val metrics TODO
-                # val_acc_metric(y_batch_val, val_logits)
-            val_loss = tf.reduce_mean(val_loss)
+                # TODO order?
+                img_width = int(
+                    tf.cast(tf.shape(images)[1], dtype=tf.float32).numpy())
+                img_height = int(
+                    tf.cast(tf.shape(images)[2], dtype=tf.float32).numpy())
+                # Set training false, so that BN uses the mean and variance of
+                # its moving statistics, learned during training.
+                val_logits = self.model(images, training=True)
+                loss = yolo_loss(
+                    labels,
+                    val_logits,
+                    self.anchors,
+                    step=step,
+                    is_training=False)
+                val_loss_avg(loss)
+
+                # Validation prediction
+                # Bounding box prediction to tensorboard.
+                tb_prediction_val = self._prediction_to_tensorboard(
+                    images, val_logits, max_outputs, step, iou_threshold,
+                    class_conf_threshold)
+                with eval_file_writer.as_default():
+                    tf.summary.image(
+                        "prediction_validation", tb_prediction_val, step=step)
+
+                # Coco mAP validation metrics.
+                # Effective batch size. Last batch could contain fewer elements.
+                eff_bs = tf.cast(
+                    tf.shape(images)[0], dtype=tf.int32).numpy().item()
+                bboxs, cat_ids, scores = self.coco_metric_format(
+                    val_logits, eff_bs, class_conf_threshold, iou_threshold,
+                    False)
+                # Iterate over all detections per image.
+                for bbox, cat_id, score in zip(bboxs, cat_ids, scores):
+                    # bbox is: [xmin,ymin,xmax,ymax]
+                    # bbox : [x,y,w,h]
+                    xmin = bbox[0]
+                    ymin = bbox[1]
+                    xmax = bbox[2]
+                    ymax = bbox[3]
+
+                    # Center coordinate.
+                    x = 0.5 * (xmin + xmax) * img_width
+                    y = 0.5 * (ymin + ymax) * img_height
+                    width = (xmax - xmin) * img_width
+                    height = (ymax - ymin) * img_height
+
+                    detect_item = {
+                        'image_id': img_id,
+                        'category_id': int(cat_id.item()),
+                        'bbox': [x, y, width, height],
+                        'score': score
+                    }
+                    detections_list.append(detect_item)
+                # Iterate iver all objects in labels.
+                bboxs, cat_ids, scores = self.coco_metric_format(
+                    labels, eff_bs, 0, iou_threshold, True)
+                obj_id = 1
+                for bbox, cat_id, score in zip(bboxs, cat_ids, scores):
+                    # bbox is: [xmin,ymin,xmax,ymax]
+                    # bbox : [x,y,w,h]
+                    xmin = bbox[0]
+                    ymin = bbox[1]
+                    xmax = bbox[2]
+                    ymax = bbox[3]
+
+                    # Center coordinate.
+                    x = 0.5 * (xmin + xmax) * img_width
+                    y = 0.5 * (ymin + ymax) * img_height
+                    width = (xmax - xmin) * img_width
+                    height = (ymax - ymin) * img_height
+
+                    grountruth_item = {
+                        'id': obj_id,
+                        'image_id': img_id,
+                        'category_id': int(cat_id.item()),
+                        'bbox': [x, y, width, height],
+                        'area': width * height,
+                        'iscrowd': 0
+                    }
+                    obj_id += 1
+                    img_list.append({"id": img_id})
+
+                    groundtruth_annotations_list.append(grountruth_item)
+
+                img_id += 1
+
+            groundtruth_dict = {
+                'annotations': groundtruth_annotations_list,
+                'images': img_list,
+                'categories': self.category_list
+            }
+            #print("gt", groundtruth_dict)
+            #print("p", detections_list)
+            metrics = compute_coco_metrics(groundtruth_dict, detections_list)
             with eval_file_writer.as_default():
-                # Todo into same graph in tb. Verbose lvl
-                tf.summary.scalar("total_loss", val_loss, step=step)
+                tf.summary.scalar(
+                    "total_loss", val_loss_avg.result(), step=step)
+                for key in metrics:
+                    tf.summary.scalar(key, metrics[key], step=step)
             eval_file_writer.flush()
+
             if mlflow:
-                mlflow.log_metric("val_loss",
-                                  val_loss.numpy(),
-                                  step=step_float)
-            # val_acc = val_acc_metric.result()
-            # val_acc_metric.reset_states()
-            # print('Validation acc: %s' % (float(val_acc), ))
-            print("...Done\n")
+                # Convert to native python type (db).
+                mlflow.log_metric(
+                    "mAP50IOU",
+                    metrics["Precision/mAP@.50IOU"].item(),
+                    step=step_float)
+                mlflow.log_metric(
+                    "AR100", metrics["Recall/AR@100"].item(), step=step_float)
+                mlflow.log_metric(
+                    "val_loss",
+                    val_loss_avg.result().numpy().item(),
+                    step=step_float)
+            print("...Done - loss {}\n".format(
+                round(val_loss_avg.result().numpy().item(), 4)))
+
+    def coco_metric_format(self, tensor, batch_size, class_conf_threshold,
+                           iou_threshold, label):
+        """[summary]
+
+        Args:
+            tensor ([type]): [description]
+
+        Returns:
+            [type]: [description]
+        """
+        bboxs = []
+        cat_ids = []
+        scores = []
+        for img in range(batch_size):
+            classes, confidence, boxes = self.decode_yolo_output(
+                tf.expand_dims(tensor[img, :, :, :, :], 0),
+                self.anchors,
+                class_conf_threshold=class_conf_threshold,
+                tb=True,
+                label=label)
+
+            # Apply multiclass NMS.
+            selected_indices = []
+            for c in range(self.n_classes):
+                # only include boxes of the current class, with > 0 confidence
+                class_mask = tf.cast(
+                    tf.equal(classes, tf.cast(c, dtype=tf.float32)),
+                    dtype=tf.float32)
+                conf_mask = tf.cast(confidence > 0, dtype=tf.float32)
+                mask = class_mask * conf_mask
+
+                # Prunes away boxes that have high intersection-over-union (IOU)
+                # overlap with previously selected boxes. Run nms independently
+                # for each cl.
+                selected_indices_per_class = tf.image.non_max_suppression(
+                    boxes, confidence * mask, 1000, iou_threshold, 0.0)
+
+                selected_indices.append(selected_indices_per_class)
+
+            # Flatten nested list.
+            selected_indices = tf.concat(selected_indices, axis=-1)
+            selected_boxes = tf.gather(boxes, selected_indices)
+            selected_classes = tf.gather(classes, selected_indices)
+            selected_confidence = tf.gather(confidence, selected_indices)
+
+            bboxs.append(selected_boxes.numpy())
+            cat_ids.append(selected_classes.numpy())
+            scores.append(selected_confidence.numpy())
+
+        # Flatten nested lists.
+        bboxs = [item for sublist in bboxs for item in sublist]
+        cat_ids = [item for sublist in cat_ids for item in sublist]
+        scores = [item for sublist in scores for item in sublist]
+        return bboxs, cat_ids, scores
 
     def restore(self, ckpt_path):
         """Restore model weights from the latest checkpoint.
@@ -330,9 +547,8 @@ class YOLO(object):
         self.ckpt_dir = os.path.join(self.logdir, "ckpts")
         if self.ckpt_manager is None:
             raise ModelNotCompiledError("use compile() first.")
-        self.ckpt_manager = tf.train.CheckpointManager(self.checkpoint,
-                                                       self.ckpt_dir,
-                                                       max_to_keep=5)
+        self.ckpt_manager = tf.train.CheckpointManager(
+            self.checkpoint, self.ckpt_dir, max_to_keep=5)
         # if a checkpoint exists, restore the latest checkpoint.
         if self.ckpt_manager.latest_checkpoint:
             self.checkpoint.restore(self.ckpt_manager.latest_checkpoint)
@@ -353,15 +569,14 @@ class YOLO(object):
         if model_format == "hdf5":
             self.model.save(os.path.join(model_dir, "yolo.h5"))
 
-        elif model_format == "SavedModel":
-            self.generator.save(os.path.join(model_dir, "yolo"))
-
     @tf.function
     def train_step(self, x, y, batch_size, step, max_outputs, file_writer,
                    lambda_coord, lambda_obj, lambda_noobj, lambda_class,
                    iou_threshold, class_confidence):
         """Decorated function (@tf.function) that creates a callable tensorflow
-        graph from a python function.
+        graph from a python function. Performs an batch inference steps,
+        calculates the gradient using BP and updates the trainable params
+        using a predefined optimization technique.
         """
         # TODO(Tobi): Trace keras.models graph to visualize in tensorboard.
         with file_writer.as_default():
@@ -388,14 +603,15 @@ class YOLO(object):
                     label=True)
 
                 # Loss
-                loss = yolo_loss(y,
-                                 prediction,
-                                 self.anchors,
-                                 lambda_coord,
-                                 lambda_obj,
-                                 lambda_noobj,
-                                 lambda_class,
-                                 step=step)
+                loss = yolo_loss(
+                    y,
+                    prediction,
+                    self.anchors,
+                    lambda_coord,
+                    lambda_obj,
+                    lambda_noobj,
+                    lambda_class,
+                    step=step)
 
             # Calculate gradient.
             gradient = tape.gradient(loss, self.model.trainable_variables)
@@ -405,7 +621,7 @@ class YOLO(object):
                 zip(gradient, self.model.trainable_variables))
 
             # Piece of shit workaround
-            return tb_prediction, tb_label
+            return tb_prediction, tb_label, loss
 
     def evaluate(self, dataset, batch_size, verbose=1):
         """TODO(): Returns loss and metrics (map, recall) for object detection.
@@ -425,7 +641,9 @@ class YOLO(object):
                            class_conf_threshold=0.25,
                            tb=False,
                            label=False):
-        """[summary] TODO for batch prediction. Only works for single prediction
+        """Performs Yolo post-processing. Extract values from yolo output tensor
+        to a readable format & filters predictions by class_conf_threshold &
+        filters predictions usind a multi-class non-max suppression.
 
         Args:
             predictions ([type]): [description]
@@ -436,7 +654,6 @@ class YOLO(object):
             tb (bool, optional): [description]. Defaults to False.
         """
 
-        batch_size = tf.cast(tf.shape(predictions)[0], dtype=tf.float32)
         # TODO wrong order?
         grid_h = tf.cast(tf.shape(predictions)[1], dtype=tf.float32)
         grid_w = tf.cast(tf.shape(predictions)[2], dtype=tf.float32)
@@ -488,8 +705,8 @@ class YOLO(object):
         boxes = tf.reshape(boxes, shape=(shape, 4))
 
         # Flatten classes
-        masked_classes = tf.cast(tf.reshape(masked_classes, shape=[shape]),
-                                 dtype=tf.float32)
+        masked_classes = tf.cast(
+            tf.reshape(masked_classes, shape=[shape]), dtype=tf.float32)
 
         # Flatten class_confidence.
         masked_confidence = tf.reshape(masked_confidence, shape=[shape])
@@ -500,8 +717,8 @@ class YOLO(object):
 
         # apply multiclass NMS
         for c in range(n_classes):
-            class_mask = tf.cast(tf.math.equal(masked_classes, c),
-                                 dtype=tf.float32)
+            class_mask = tf.cast(
+                tf.math.equal(masked_classes, c), dtype=tf.float32)
             score_mask = tf.cast(masked_confidence > 0, dtype=tf.float32)
             mask = class_mask * score_mask
             # Prunes away boxes that have high intersection-over-union (IOU)
@@ -522,7 +739,7 @@ class YOLO(object):
         return selected_boxes, selected_confidence, selected_classes
 
     def _prediction_to_tensorboard(self,
-                                   input,
+                                   input_img,
                                    prediction,
                                    max_outputs,
                                    step,
@@ -542,9 +759,9 @@ class YOLO(object):
             selected_indices = []
             for c in range(self.n_classes):
                 # only include boxes of the current class, with > 0 confidence
-                class_mask = tf.cast(tf.equal(classes,
-                                              tf.cast(c, dtype=tf.float32)),
-                                     dtype=tf.float32)
+                class_mask = tf.cast(
+                    tf.equal(classes, tf.cast(c, dtype=tf.float32)),
+                    dtype=tf.float32)
                 conf_mask = tf.cast(confidence > 0, dtype=tf.float32)
                 mask = class_mask * conf_mask
 
@@ -559,10 +776,11 @@ class YOLO(object):
             # Flatten nested list.
             selected_indices = tf.concat(selected_indices, axis=-1)
             selected_boxes = tf.gather(boxes, selected_indices)
-            selected_classes = tf.gather(classes, selected_indices)
-            selected_confidence = tf.gather(confidence, selected_indices)
-            selected_boxes = tf.cast(tf.expand_dims(selected_boxes, 0),
-                                     dtype=tf.float32)
+            # TODO draw_boundin_boxes does not support labels right now.
+            #selected_classes = tf.gather(classes, selected_indices)
+            #selected_confidence = tf.gather(confidence, selected_indices)
+            selected_boxes = tf.cast(
+                tf.expand_dims(selected_boxes, 0), dtype=tf.float32)
 
             # Swap xy coordinates for bullshit tensorflow draw_bounding_boxes.
             x1 = selected_boxes[..., 0:1]
@@ -572,8 +790,9 @@ class YOLO(object):
             boxes_yx = tf.concat([y1, x1, y2, x2], axis=-1)
 
             tb_img = tf.image.draw_bounding_boxes(
-                tf.cast(tf.expand_dims(input[img, :, :, :], 0),
-                        dtype=tf.float32), boxes_yx, self.colors)
+                tf.cast(
+                    tf.expand_dims(input_img[img, :, :, :], 0),
+                    dtype=tf.float32), boxes_yx, self.colors)
             tb_imgs.append(tb_img)
 
         tb_output = tf.concat(tb_imgs, axis=0)
