@@ -1,7 +1,4 @@
-"""
-
-"""
-import numpy as np
+"""Implements the yolov2 loss."""
 import tensorflow as tf
 
 
@@ -71,8 +68,8 @@ def yolo_loss(y_true,
     iou_scores = _calculate_ious(y_true, pred)
     # Calcculate max IOUs TODO
     num_true_labels = grid_h * grid_w * boxes
-    y_true_p = tf.reshape(
-        y_true[..., :4], shape=(batch_size, 1, 1, 1, num_true_labels, 4))
+    y_true_p = tf.reshape(y_true[..., :4],
+                          shape=(batch_size, 1, 1, 1, num_true_labels, 4))
 
     # (bs, grid, grid, boxes, num_true_labels)
     iou_scores_buff = _calculate_ious(y_true_p, tf.expand_dims(y_pred, axis=4))
@@ -118,10 +115,9 @@ def yolo_grid(batch_size, grid_h, grid_w, boxes):
         [Tensor]: Coordinate grid values.
     """
     # TODO eval
-    cell_x = tf.cast(
-        tf.reshape(
-            tf.tile(tf.range(grid_w), [grid_h]), (1, grid_h, grid_w, 1, 1)),
-        dtype=tf.float32)
+    cell_x = tf.cast(tf.reshape(tf.tile(tf.range(grid_w), [grid_h]),
+                                (1, grid_h, grid_w, 1, 1)),
+                     dtype=tf.float32)
 
     # TODO non symmetrie
     cell_y = tf.transpose(cell_x, (0, 2, 1, 3, 4))
@@ -130,8 +126,8 @@ def yolo_grid(batch_size, grid_h, grid_w, boxes):
     ##    cell_grid[n, i, j, anchor, k] = j when k = 0
     ## for any n, k, i, j
     ##    cell_grid[n, i, j, anchor, k] = i when k = 1
-    cell_grid = tf.tile(
-        tf.concat([cell_x, cell_y], -1), [batch_size, 1, 1, boxes, 1])
+    cell_grid = tf.tile(tf.concat([cell_x, cell_y], -1),
+                        [batch_size, 1, 1, boxes, 1])
 
     return cell_grid
 
@@ -168,9 +164,8 @@ def extract_model_output(y_pred, grid_coord, anchors):
     # Unrestricted w, h values.
     box_wh = y_pred[..., 2:4]
     # Make w, h strictly positive values. TODO check
-    box_wh = tf.cast(
-        tf.exp(box_wh), dtype=tf.float32) * tf.cast(
-            tf.reshape(anchors, [1, 1, 1, boxes, 2]), dtype=tf.float32)
+    box_wh = tf.cast(tf.exp(box_wh), dtype=tf.float32) * tf.cast(
+        tf.reshape(anchors, [1, 1, 1, boxes, 2]), dtype=tf.float32)
 
     # Restrict box_confidence values to [0, 1].
     # Shape (bs, grid, grid, anchors, 1)
@@ -246,8 +241,8 @@ def _loss_xywh(true_box_xy, pred_box_xy, true_box_wh, pred_box_wh,
     loss_xy = tf.reduce_sum(
         tf.square(
             tf.cast(true_box_xy, dtype=tf.float32) -
-            tf.cast(pred_box_xy, dtype=tf.float32)) * tf.cast(
-                coord_mask, dtype=tf.float32))
+            tf.cast(pred_box_xy, dtype=tf.float32)) *
+        tf.cast(coord_mask, dtype=tf.float32))
     # Use sqrt of width and height to weigh small deviations in large boxes
     # less than in small boxes.
     loss_wh = tf.reduce_sum(
@@ -282,8 +277,8 @@ def _loss_class(
     loss_class = tf.nn.sparse_softmax_cross_entropy_with_logits(
         labels=true_box_class, logits=p_box_class)
     loss_class = tf.reduce_sum(
-        tf.cast(loss_class, dtype=tf.float32) * tf.cast(
-            class_mask, dtype=tf.float32))
+        tf.cast(loss_class, dtype=tf.float32) *
+        tf.cast(class_mask, dtype=tf.float32))
     return loss_class
 
 
